#include "glxbackend.h"

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <GL/glxext.h>
#include <string>

/* attributes for a double buffered visual in RGBA format with at least
* 4 bits per color */                       
static int attrList[] =                                             
{                                                                      
	GLX_RGBA, GLX_DOUBLEBUFFER,
	GLX_RED_SIZE, 4,                                                   
	GLX_GREEN_SIZE, 4,                                                 
	GLX_BLUE_SIZE, 4,
	None                                                               
};

VisualID GLXBackend::initialize(Display *display)
{
	if (this->display == NULL) { // this implies that the context is also unitialized
		this->display = display;
		// debugging: show version information
		int glxMajor, glxMinor;
		glXQueryVersion(display, &glxMajor, &glxMinor);
		printf("Using GLX version %d.%d\n", glxMajor, glxMinor);
		// check for swap control functions
		const char *extensions = glXQueryExtensionsString(display, DefaultScreen(display));
		if (std::string(extensions).find("GLX_MESA_swap_control") != std::string::npos) {
			funSwapIntervalMesa = (PFNGLXSWAPINTERVALMESAPROC)glXGetProcAddress((const GLubyte*)"glXSwapIntervalMESA");
			if (funSwapIntervalMesa) printf("glXSwapIntervalMESA is present\n");
		}
		else {
			funSwapIntervalMesa = 0;
		}
		if (std::string(extensions).find("GLX_EXT_swap_control") != std::string::npos) {
			funSwapIntervalExt = (PFNGLXSWAPINTERVALEXTPROC)glXGetProcAddress((const GLubyte*)"glXSwapIntervalEXT");
			if (funSwapIntervalExt) printf("glXSwapIntervalEXT is present\n");
		}
		else {
			funSwapIntervalExt = 0;
		}
		// get the visual
		vi = glXChooseVisual(display, DefaultScreen(display), attrList);
	}
	return vi->visualid;
}

void GLXBackend::createContext(Window window)
{
	assert(display != NULL && context == None);
	this->window = window;
	// create context with that window
	context = glXCreateContext(display, vi, 0, GL_TRUE);                             
	glXMakeCurrent(display, window, context);                                      
	assert(glXIsDirect(display, context));
}

GLXBackend::~GLXBackend()
{
	if (display != NULL) {
		if (context != None) {
			glXMakeCurrent(display, None, NULL);
			glXDestroyContext(display, context);
		}
		XFree(vi);
	}
}

void GLXBackend::swapBuffers() const
{
	assert(context != None); // this implies the display is also initialized
	glXSwapBuffers(display, window);
}

void GLXBackend::setSwapInterval(int i) const
{
	assert(context != None);
	if (funSwapIntervalExt)
		funSwapIntervalExt(display, window, i);
	else if (funSwapIntervalMesa)
		funSwapIntervalMesa(i);
	else {
		fprintf(stderr, "At least one of glXSwapIntervalMESA, glXSwapIntervalEXT must be provided by the system\n");
		abort();
	}
}
